<?php

namespace Drupal\token_eca_alter\Service;

use Drupal\token\Token;
use Drupal\token_eca_alter\Events\TokenAlterEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * The token alter process.
 *
 * @package Drupal\token_eca_alter\Service
 */
class TokenAlterProcess {


  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected EventDispatcherInterface $eventDispatcher;

  /**
   * The token.
   *
   * @var \Drupal\token\Token
   */
  protected Token $token;

  /**
   * Set the dispatcher service.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher.
   */
  public function setDispatcher(EventDispatcherInterface $eventDispatcher) {
    $this->eventDispatcher = $eventDispatcher;
  }

  /**
   * Set the token.
   *
   * @param \Drupal\token\Token $token
   *   The token service.
   */
  public function setToken(Token $token) {
    $this->token = $token;
  }

  /**
   * Process the alter token service.
   *
   * @param string $name
   *   The token name.
   * @param array $data
   *   The token data.
   * @param array $options
   *   The token options.
   *
   * @return string
   *   Processed token.
   */
  public function processToken(string $name, array $data, array $options): string {
    if ($pos_e = strpos($name, '}')) {
      $alter_info = substr($name, 0, $pos_e + 1);
      $first = 1;
      $token = str_replace($alter_info . ':', '', $name, $first);
      $post_s = strpos($name, '{');
      $alter_id = substr($alter_info, 0, $post_s);
      $alter_options = str_replace([$alter_id, '{', '}'], '', $alter_info);
      parse_str(str_replace(':', '=', $alter_options), $alter_options);
    }
    else {
      $alter_options = [];
      $parts = explode(':', $name);
      $alter_id = array_shift($parts);
      $token = implode(':', $parts);
    }
    $data = $this->token->replace('[' . $token . ']', $data, $options + ['clear' => TRUE]);
    $tokenAlterEvent = new TokenAlterEvent($alter_id, $token, $data, $alter_options);
    $this->eventDispatcher->dispatch($tokenAlterEvent, TokenAlterEvent::ALTER);
    return $tokenAlterEvent->getData();
  }

}
